# This file is autogenerated
import threading
import ctypes
import os
from .autd3capi_def import Level, LinkPtr, TimerStrategy

from enum import IntEnum

class SyncMode(IntEnum):
    FreeRun = 0
    DC = 1

    @classmethod
    def from_param(cls, obj):
        return int(obj)


class Singleton(type):
    _instances = {}  # type: ignore
    _lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            with cls._lock:
                if cls not in cls._instances:
                    cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class NativeMethods(metaclass=Singleton):

    def init_dll(self, bin_location: str, bin_prefix: str, bin_ext: str):
        try:
            self.dll = ctypes.CDLL(os.path.join(bin_location, f'{bin_prefix}autd3capi_link_soem{bin_ext}'))
        except Exception:
            return

        self.dll.AUTDAdapterPointer.argtypes = [] 
        self.dll.AUTDAdapterPointer.restype = ctypes.c_void_p

        self.dll.AUTDAdapterGetSize.argtypes = [ctypes.c_void_p] 
        self.dll.AUTDAdapterGetSize.restype = ctypes.c_uint32

        self.dll.AUTDAdapterGetAdapter.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.c_char_p, ctypes.c_char_p] 
        self.dll.AUTDAdapterGetAdapter.restype = None

        self.dll.AUTDAdapterPointerDelete.argtypes = [ctypes.c_void_p] 
        self.dll.AUTDAdapterPointerDelete.restype = None

        self.dll.AUTDLinkSOEM.argtypes = [] 
        self.dll.AUTDLinkSOEM.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithSendCycle.argtypes = [LinkPtr, ctypes.c_uint16]  # type: ignore 
        self.dll.AUTDLinkSOEMWithSendCycle.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithSync0Cycle.argtypes = [LinkPtr, ctypes.c_uint16]  # type: ignore 
        self.dll.AUTDLinkSOEMWithSync0Cycle.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithBufSize.argtypes = [LinkPtr, ctypes.c_uint32]  # type: ignore 
        self.dll.AUTDLinkSOEMWithBufSize.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithTimerStrategy.argtypes = [LinkPtr, TimerStrategy]  # type: ignore 
        self.dll.AUTDLinkSOEMWithTimerStrategy.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithSyncMode.argtypes = [LinkPtr, SyncMode]  # type: ignore 
        self.dll.AUTDLinkSOEMWithSyncMode.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithIfname.argtypes = [LinkPtr, ctypes.c_char_p]  # type: ignore 
        self.dll.AUTDLinkSOEMWithIfname.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithStateCheckInterval.argtypes = [LinkPtr, ctypes.c_uint32]  # type: ignore 
        self.dll.AUTDLinkSOEMWithStateCheckInterval.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithOnLost.argtypes = [LinkPtr, ctypes.c_void_p]  # type: ignore 
        self.dll.AUTDLinkSOEMWithOnLost.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithLogLevel.argtypes = [LinkPtr, Level]  # type: ignore 
        self.dll.AUTDLinkSOEMWithLogLevel.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithLogFunc.argtypes = [LinkPtr, ctypes.c_void_p, ctypes.c_void_p]  # type: ignore 
        self.dll.AUTDLinkSOEMWithLogFunc.restype = LinkPtr

        self.dll.AUTDLinkSOEMWithTimeout.argtypes = [LinkPtr, ctypes.c_uint64]  # type: ignore 
        self.dll.AUTDLinkSOEMWithTimeout.restype = LinkPtr

        self.dll.AUTDLinkRemoteSOEM.argtypes = [ctypes.c_char_p, ctypes.c_char_p] 
        self.dll.AUTDLinkRemoteSOEM.restype = LinkPtr

        self.dll.AUTDLinkRemoteSOEMWithTimeout.argtypes = [LinkPtr, ctypes.c_uint64]  # type: ignore 
        self.dll.AUTDLinkRemoteSOEMWithTimeout.restype = LinkPtr

    def adapter_pointer(self) -> ctypes.c_void_p:
        return self.dll.AUTDAdapterPointer()

    def adapter_get_size(self, adapters: ctypes.c_void_p) -> ctypes.c_uint32:
        return self.dll.AUTDAdapterGetSize(adapters)

    def adapter_get_adapter(self, adapters: ctypes.c_void_p, idx: int, desc: ctypes.Array[ctypes.c_char], name: ctypes.Array[ctypes.c_char]) -> None:
        return self.dll.AUTDAdapterGetAdapter(adapters, idx, desc, name)

    def adapter_pointer_delete(self, adapters: ctypes.c_void_p) -> None:
        return self.dll.AUTDAdapterPointerDelete(adapters)

    def link_soem(self) -> LinkPtr:
        return self.dll.AUTDLinkSOEM()

    def link_soem_with_send_cycle(self, soem: LinkPtr, cycle: int) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithSendCycle(soem, cycle)

    def link_soem_with_sync_0_cycle(self, soem: LinkPtr, cycle: int) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithSync0Cycle(soem, cycle)

    def link_soem_with_buf_size(self, soem: LinkPtr, buf_size: int) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithBufSize(soem, buf_size)

    def link_soem_with_timer_strategy(self, soem: LinkPtr, timer_strategy: TimerStrategy) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithTimerStrategy(soem, timer_strategy)

    def link_soem_with_sync_mode(self, soem: LinkPtr, mode: SyncMode) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithSyncMode(soem, mode)

    def link_soem_with_ifname(self, soem: LinkPtr, ifname: bytes) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithIfname(soem, ifname)

    def link_soem_with_state_check_interval(self, soem: LinkPtr, interval_ms: int) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithStateCheckInterval(soem, interval_ms)

    def link_soem_with_on_lost(self, soem: LinkPtr, on_lost_func: ctypes.c_void_p) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithOnLost(soem, on_lost_func)

    def link_soem_with_log_level(self, soem: LinkPtr, level: Level) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithLogLevel(soem, level)

    def link_soem_with_log_func(self, soem: LinkPtr, out_func: ctypes.c_void_p, flush_func: ctypes.c_void_p) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithLogFunc(soem, out_func, flush_func)

    def link_soem_with_timeout(self, soem: LinkPtr, timeout_ns: int) -> LinkPtr:
        return self.dll.AUTDLinkSOEMWithTimeout(soem, timeout_ns)

    def link_remote_soem(self, addr: bytes, err: ctypes.Array[ctypes.c_char]) -> LinkPtr:
        return self.dll.AUTDLinkRemoteSOEM(addr, err)

    def link_remote_soem_with_timeout(self, soem: LinkPtr, timeout_ns: int) -> LinkPtr:
        return self.dll.AUTDLinkRemoteSOEMWithTimeout(soem, timeout_ns)
